package info.msxlaunchers.openmsx.launcher.ui.presenter;

import info.msxlaunchers.openmsx.launcher.data.settings.Settings;
import info.msxlaunchers.openmsx.launcher.data.settings.constants.Language;
import info.msxlaunchers.openmsx.launcher.persistence.settings.SettingsPersister;
import info.msxlaunchers.openmsx.launcher.ui.presenter.LauncherException;
import info.msxlaunchers.openmsx.launcher.ui.view.SettingsView;

import java.io.IOException;
import java.util.HashSet;
import java.util.Set;

import org.junit.Before;
import org.junit.Test;

import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;

import static org.mockito.Matchers.any;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;

public class SettingsPresenterImplTest
{
	@Mock SettingsView settingsView;
	@Mock SettingsPersister settingsPersister;
	@Mock MainPresenter mainPresenter;

	@Before
	public void prepare()
	{
		MockitoAnnotations.initMocks( this );
	}

	@Test
	public void testConstructor() throws IOException
	{
		new SettingsPresenterImpl( settingsView, settingsPersister, mainPresenter );
	}

	@Test( expected = NullPointerException.class )
	public void testConstructorArg1Null() throws IOException
	{
		new SettingsPresenterImpl( null, settingsPersister, mainPresenter );
	}

	@Test( expected = NullPointerException.class )
	public void testConstructorArg2Null() throws IOException
	{
		new SettingsPresenterImpl( settingsView, null, mainPresenter );
	}

	@Test( expected = NullPointerException.class )
	public void testConstructorArg3Null() throws IOException
	{
		new SettingsPresenterImpl( settingsView, settingsPersister, null );
	}

	@Test
	public void testOnRequestSettingsScreen() throws IOException
	{
		SettingsPresenterImpl presenter = new SettingsPresenterImpl( settingsView, settingsPersister, mainPresenter );

		String defaultDatabase = "database";
		Set<String> databases = new HashSet<String>();
		Language language = Language.FRENCH;
		String languageCode = "FR";

		Settings settings = new Settings( null, null, null, defaultDatabase, null );

		presenter.onRequestSettingsScreen( settings, databases, language, languageCode, false );

		verify( settingsView, times( 1 ) ).displaySettings( settings, databases, language, languageCode, false );
	}

	@Test
	public void testOnRequestSettingsAction() throws IOException, LauncherException
	{
		SettingsPresenterImpl presenter = new SettingsPresenterImpl( settingsView, settingsPersister, mainPresenter );

		String openMSXPath = "openMSXPath";
		String machinesPath = "machinesPath";
		String screenshotsPath = "screenshotsPath";
		String defaultDatabase = "database";
		String language = "ENGLISH";

		Settings settings = new Settings( openMSXPath, machinesPath, screenshotsPath, defaultDatabase, Language.valueOf( language ) );

		presenter.onRequestSettingsAction( openMSXPath, machinesPath, screenshotsPath, defaultDatabase, language );

		verify( settingsPersister, times( 1 ) ).saveSettings( settings );
		verify( mainPresenter, times( 1 ) ).onAcceptSettingsAction( settings );		
	}

	@Test
	public void testOnRequestSettingsActionNullLanguage() throws IOException, LauncherException
	{
		SettingsPresenterImpl presenter = new SettingsPresenterImpl( settingsView, settingsPersister, mainPresenter );

		String openMSXPath = "openMSXPath";
		String machinesPath = "machinesPath";
		String screenshotsPath = "screenshotsPath";
		String defaultDatabase = "database";

		Settings settings = new Settings( openMSXPath, machinesPath, screenshotsPath, defaultDatabase, null );

		presenter.onRequestSettingsAction( openMSXPath, machinesPath, screenshotsPath, defaultDatabase, null );

		verify( settingsPersister, times( 1 ) ).saveSettings( settings );
		verify( mainPresenter, times( 1 ) ).onAcceptSettingsAction( settings );		
	}

	@Test( expected = LauncherException.class )
	public void testOnRequestSettingsActionIOException() throws IOException, LauncherException
	{
		SettingsPresenterImpl presenter = new SettingsPresenterImpl( settingsView, settingsPersister, mainPresenter );

		String openMSXPath = "openMSXPath";
		String machinesPath = "machinesPath";
		String screenshotsPath = "screenshotsPath";
		String defaultDatabase = "database";

		Mockito.doThrow( new IOException() ).when( settingsPersister ).saveSettings( (Settings)any() );

		presenter.onRequestSettingsAction( openMSXPath, machinesPath, screenshotsPath, defaultDatabase, null );
	}
}