/*
 * Copyright 2014 Sam Elsharif
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package info.msxlaunchers.openmsx.launcher.ui.view.platform;

import info.msxlaunchers.openmsx.launcher.ui.presenter.MainPresenter;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

import javax.swing.UIManager;

/**
 * Mac implementation of <code>PlatformViewProperties</code>
 * 
 * @since v1.2
 * @author Sam Elsharif
 *
 */
final class MacViewProperties implements PlatformViewProperties
{
	private final MainPresenter mainPresenter;

	MacViewProperties(MainPresenter mainPresenter)
	{
		this.mainPresenter = mainPresenter;
	}

	/* (non-Javadoc)
	 * @see info.msxlaunchers.openmsx.launcher.ui.view.platform.PlatformViewProperties#isCanSuggestOpenMSXPathAndMachinesPath()
	 */
	@Override
	public boolean isCanSuggestOpenMSXPathAndMachinesPath()
	{
		return true;
	}

	/* (non-Javadoc)
	 * @see info.msxlaunchers.openmsx.launcher.ui.view.platform.PlatformViewProperties#getSuggestedOpenMSXPath()
	 */
	@Override
	public String getSuggestedOpenMSXPath()
	{
		return "/Applications/openmsx.app/Contents/MacOS";
	}

	/* (non-Javadoc)
	 * @see info.msxlaunchers.openmsx.launcher.ui.view.platform.PlatformViewProperties#getSuggestedOpenMSXMachinesPath()
	 */
	@Override
	public String getSuggestedOpenMSXMachinesPath()
	{
		return "/Applications/openmsx.app/share/machines";
	}

	/* (non-Javadoc)
	 * @see info.msxlaunchers.openmsx.launcher.ui.view.platform.PlatformViewProperties#setDisplayProperties()
	 */
	@Override
	public void setDisplayProperties()
	{
		//take the menu bar off the JFrame
		System.setProperty("apple.laf.useScreenMenuBar", "true");

		//set the name of the application menu item
		System.setProperty("com.apple.mrj.application.apple.menu.about.name", "openMSX Launcher");

		//set the look and feel
		try
		{
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		}
		catch(Exception e)
		{
			//should not happen
		}

		//define action for the Mac About in the application menu
	    try
	    {
	        Object app = Class.forName("com.apple.eawt.Application").getMethod("getApplication", (Class[]) null).invoke(null, (Object[]) null);

	        Object al = Proxy.newProxyInstance(Class.forName("com.apple.eawt.AboutHandler").getClassLoader(),
	        				new Class[] { Class.forName("com.apple.eawt.AboutHandler") },
	        				new AboutListener());
	        app.getClass().getMethod("setAboutHandler", new Class[] { Class.forName("com.apple.eawt.AboutHandler") }).invoke(app, new Object[] { al });
	    }
	    catch (Exception e)
	    {
	        //shouldn't get here
	    }
	}

	private class AboutListener implements InvocationHandler
	{
	    public Object invoke(Object proxy, Method method, Object[] args)
	    {
	        //Show About Dialog
	    	mainPresenter.onRequestAboutScreen();

	        return null;
	    }
	}
}
